;markg10m.asm - a program to generate 250, 100, 50, 25, 10 and 5 Khz markers
;       Derived from the markrgen program - changed the loops to account
;       for a 10 MHz clock instead of the markrgen 4 MHz clock and added
;       the 250 KHz loop.
;
; 10/27/99 Chuck Olson, WB9KZY
;       Jackson Harbor Press
;       http://jacksonharbor.home.att.net
;       jacksonharbor@att.net
;
;
;       VVHATDOESITDO?
;
;  This program uses a 16F84 PIC microcontroller to generate accurate
; square waves at frequencies of 250, 100, 50, 25, 10 and 5 KHz.  The output
; is coupled via a very small value cap to a receiver, the odd and even 
; harmonics of these markers can be heard all the way through to 30 MHz.   
;  The markers are selected via a momentary push button switch closure to
; ground on pin 1.  At powerup, the PIC will output a 250 KHz signal
; to pin 6.  If the momentary switch is pressed and released (PAR), the
; the output will change to 100 KHz - subsequent PARs will advance down
; to the 5 KHz marker and then wraparound back to the 250 KHz marker.
; 
; If pin 2 is grounded, the marker will be gated (turned on and off at
; a regular interval).  The frequency of the gating will increase as 
; the user changes the markers to allow easy identification of the marker.
; The 100 KHz marker has a slow on/off tone while the 5 KHz marker will
; have a very rapid buzz.
; 
; If pin 18 is grounded, the duty cycle will change from the normal 50%
; square wave to a 10% square wave.  The idea was to even out the even
; and odd harmonic amplitudes, however, this isn't really accomplished
; but decided to leave it in anyway.  It may (depending on the frequency)
; increase the amplitude of the even harmonics slightly.
; 
; Programming notes:
; 
;  The program is basically a timed loop.  The 250 KHz loop is the
; tightest, executing in 4 microseconds (us).  With a 10 MHz clock the
; PIC executes all instructions in either .4 or .8 us. 
; The cycles of the loop have been counted to allow for exactly the
; correct time delay - the PIC output is then set to a 1 or 0 as needed.
; Seperate loops are used for all 6 markers and duty cycles - there are
; undoubtedly more efficient ways to do this, but this method does work.
; 
; The gating of the markers is accomplished with the use of the watch dog
; timer (WDT) feature of the PIC.  The WDT is a seperate RC on-chip
; oscillator with a period of about 18 ms.  The prescaler is used to count
; an integer number of WDT periods - after the prescaler overflows, the
; chip is reset, then a matching period of silence is generated using a
; timed loop.  The loop to be executed is determined from the bits of
; the FSR register - the FSR register was used because it is left alone
; during a reset of the chip.
; 
;  Hardware notes:
; 
;  I built a prototype on perf board  using a 4 MHz rated part - one of the
; two 33 pf caps connected to the crystal should be variable trimmer
; cap to allow this adjustment.  It should be possible to use an external
; 10 MHz oscillator to drive pin 16 instead of using a crystal although
; I haven't actually tried this.
; 
;  PIC pinout:
;
;pin    name    function/connection
;---    ----    -----------------------------------------------
;  1    ra2     marker select, pulled up to +5V with a 10K resistor
;  2    ra3     gate select, pulled up to +5V with a 10 k resistor
;  3    ra4     no connect
;  4    MCLR    pulled up to +5V with a 10K resistor
;  5    VSS     Ground 
;  6    rb0     marker output
;  7    rb1     no connect
;  8    rb2     no connect
;  9    rb3     no connect
; 10    rb4     no connect
; 11    rb5     no connect
; 12    rb6     no connect
; 13    rb7     no connect
; 14    VDD     +5V
; 15    osc2    connected to one side of 10 MHz xtal, also 33 pf cap to gnd
; 16    osc1    connected to other side of 10 MHz crystal & 33 pf cap to gnd
; 17    ra0     no connect
; 18    ra1     duty cycle select, pulled up to +5V with a 10 k resistor


	LIST P=16F84, R=DEC 
	INCLUDE "p16F84.inc"
	__FUSES _PWRTE_OFF & _CP_OFF & _WDT_ON & _HS_OSC        

;constants
; bit definitions
freq    equ     d'176'   ;1/2 ms delay loop constant


;  bit usage in the register FSR   -   bit 0 = 1 -> 100 KHz
;                                      bit 1 = 1 ->  50 KHz
;                                      bit 2 = 1 ->  25 KHz
;                                      bit 3 = 1 ->  10 KHz
;                                      bit 4 = 1 ->   5 KHz
;                                      bit 5 = 1 -> 250 KHz


; memory locations
count   equ     0x0C    ;a delay loop counter variable
temp    equ     0x0D    ;used in the delay loops
temp2   equ     0x0E    ;used in the delay loops
temp3   equ     0x0F    ;used in the delay loops


	ORG     0x000
prestart  
	goto  init      ;reset vector
	

	ORG     0x004
noint   goto  init      ;interrupt vector (shouldn't be needed)


; halfms - a routine to wait roughly 1/2 millisecond
;delay = call/setup/return(7)+(7*176)-1 * .4 us = 495 us
halfms  movlw   freq    ;load up the delay constant
	movwf   temp    ;store it in the temporary location
	;10/27/99 added next 2 gotos to account for 2.5x speed at 10 MHz
	;max delay in a simple loop at 10 MHz is only 3*255*.4 us = 306 us
loopy   goto    loopy1  ;waste 2 cycles
loopy1  goto    loopy2  ;waste 2 more cycles
loopy2  decfsz  temp,1  ;decrement the temp location - is it 0 ?
	goto    loopy   ;no, keep decrementing
	;yes, return to the calling routine
	clrwdt          ;clear the watchdog timer
	retlw   0       ;end of half cycle routine


; deboun2 - a routine to wait roughly 20 ms
;       10/27/99 - added deboun2 to enter the deboun routine for only
;               a 20 ms delay
deboun2 movlw   d'40'   ;load up a counter for 20 ms (40 x .5 ms)
	goto    db2     ;enter the deboun routine
; deboun - a routine to wait roughly 40 ms
;       10/27/99 - changed deboun to a 40 ms delay to 
;               eliminate need to call deboun twice in a row several times
deboun  movlw   d'80'   ;load up a counter for 40 ms (80 x .5 ms)
db2     movwf   temp2   ;store it in the second temp location
loopdb  decfsz  temp2,1 ;decrement the dit length counter - is it 0 ?
	goto    ckdel   ;no, start checking and waiting
	goto    dbdun   ;yes, you're done
ckdel   ;call    padchk  ;get the paddle status (13 us)
	call    halfms  ;wait a .5 ms
	goto    loopdb  ;continue
dbdun   retlw   0       ;end of the debounce routine


; init - the initialization power up entry point of the program
init    bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00000000'     ; all outputs on PORTB
	movwf   TRISB           ; set the tri-state register
	movlw   b'00001110'     ; RA1,2,3 are inputs
	movwf   TRISA           ; set the tri-state register
	movlw   b'00001101'     ; set WDT for 32x (101) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
waithlf btfsc   FSR,0  ;test for 100 khz
	movlw   d'16'   ;load up a .32 second delay (.02 x 16)
	btfsc   FSR,1  ;test for 50 khz
	movlw   d'8'   ;load up a .16 second delay (.02 x 8)
	btfsc   FSR,2  ;test for 25 khz
	movlw   d'4'   ;load up a .08 second delay (.02 x 4)
	btfsc   FSR,3  ;test for 10 khz
	movlw   d'2'   ;load up a .04 second delay (.02 x 2)
	btfsc   FSR,4  ;test for 5 khz
	movlw   d'1'   ;load up a .02 second delay (.02 x 1)
	btfsc   FSR,5  ;test for 250 khz
	movlw   d'32'   ;load up a .64 second delay (.02 x 32)
	movwf   temp3   ;store it in the temporary location
loopx   call    deboun2 ;wait 20 ms to debounce the switch
	decfsz  temp3,1 ;decrement the temp location - is it 0 ?
	goto    loopx   ;no, keep decrementing
	;yes, see which loop we are in
	btfsc   FSR,5  ;test for 250 khz
	goto    strt2501        ;start with 250 KHz
	btfsc   FSR,0  ;test for 100 khz
	goto    strt1001        ;start with 100 KHz
	btfsc   FSR,1  ;test for 50 khz
	goto    strt501         ;start with 50 KHz
	btfsc   FSR,2  ;test for 25 khz
	goto    strt251         ;start with 25 KHz
	btfsc   FSR,3  ;test for 10 khz
	goto    strt101         ;start with 10 KHz
	btfsc   FSR,4  ;test for 5 khz
	goto    start51         ;start with 5 KHz


; rot250 - a routine to select the next marker frequency
rot250  ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rot250  ;no, wait another delay time
			;yes, drift into the 250 Khz loop


;  250 khz loop
;       2 usec high, 2 usec low
; 10/27/99 - use the 4 MHz (100KHz loop) code at 10 MHz to get a 250 KHz
;       marker (10 cycles x .4 us/cycle = 2 usec hi/lo times)
strt2501                        
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001110'     ; set WDT for 64x (110) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x20    ;load up the flag for 250 khz
	movwf   FSR    ;store it in loop
strt250 bsf     PORTB,0         ;1  1  set output (pin  6) high
	btfss   PORTA,3         ;2  3 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2  5 has the button on pin 18 been pressed?
	goto    estr250         ;   it's pressed, enter the 10% duty cycle loop
	bcf     PORTB,0         ;1  6  set output (pin  6) low
	btfss   PORTA,2         ;2  8 has the button on pin 1 been pressed?
	goto    rotate          ;yes, bail to the rotate to a new loop routine
	goto    strt250         ;2 10  keep at it


; 10 % duty cycle 250 khz loop
estr250 bsf     PORTB,0         ;1  1  set output (pin  6) high
	bcf     PORTB,0         ;1  2  set output (pin  6) low
	btfss   PORTA,3         ;2  4 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2  6 has the button on pin 18 been released?
	goto    strt250         ;  it's released, enter the 50% duty cycle loop
	btfss   PORTA,2         ;2  8 has the button on pin 1 been pressed?
	goto    rotate          ;yes, bail to the rotate to a new loop routine
	goto    estr250         ;2 10  keep at it


; rotate - a routine to select the next marker frequency
rotate  ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rotate  ;no, wait another delay time
			;yes, drift into the 100 Khz loop


;  100 khz loop
;       PORTB bit 0, 5 usec high, 5 usec low
; 10/27/99 at 10 MHz, can't make the hi/lo exactly 5 usec (5/.4=12.5) so
;       made the output slightly off center, 4.8 hi/5.2 lo  - 12 hi / 13 lo
;also, now need 25 total cycles in loop instead of 10
strt1001                        
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001101'     ; set WDT for 32x (101) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x01    ;load up the flag for 100 khz
	movwf   FSR    ;store it in loop
strt100 bsf     PORTB,0         ;1  1  set output (pin  6) high
	btfss   PORTA,3         ;2  3 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2  5 has the button on pin 18 been pressed?
	goto    estr100         ;   it's pressed, enter the 10% duty cycle loop
	goto    s100ka          ;2  7 waste 2 cycles
s100ka  goto    s100kb          ;2  9 waste another 2 cycles
s100kb  goto    s100kc          ;2 11 waste yet another 2 cycles
s100kc  nop                     ;1 12 waste a cycle
	bcf     PORTB,0         ;1 13  set output (pin  6) low
	btfss   PORTA,2         ;2 15 has the button on pin 1 been pressed?
	goto    rotat50         ;yes, bail to the rotate to a new loop routine
	goto    s100kd          ;2 17 waste another 2 cycles
s100kd  goto    s100ke          ;2 19 waste yet another 2 cycles
s100ke  goto    s100kf          ;2 21 waste a cycle
s100kf  goto    s100kg          ;2 23 waste yet another 2 cycles
s100kg  goto    strt100         ;2 25  keep at it


; less than 10 % duty cycle 100 khz loop
estr100 bsf     PORTB,0         ;1  1  set output (pin  6) high
	nop                     ;1  2  waste a cycle
	bcf     PORTB,0         ;1  3  set output (pin  6) low
	btfss   PORTA,3         ;2  5 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2  7 has the button on pin 18 been released?
	goto    strt100         ;  it's released, enter the 50% duty cycle loop
	btfss   PORTA,2         ;2  9 has the button on pin 1 been pressed?
	goto    rotat50         ;yes, bail to the rotate to a new loop routine
	movlw   4               ;1 10  load up a loop count
	movwf   count           ;1 11  save it 
	;total CYCLE count of loop is 3 x 3 + 2 = 11
es100kh decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    es100kh         ;2 22  loop back
	nop                     ;1 23  waste a cycle
	goto    estr100         ;2 25  keep at it

	
; rotat50 - a routine to select the next marker frequency
rotat50 ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rotat50 ;no, wait another delay time
			;yes, drift into the 50 Khz loop


;  50 khz loop
;       PORTB bit 0, 10 usec high, 10 usec low (25 cycles hi/lo)
strt501 
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001100'     ; set WDT for 16x (100) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x02    ;load up the flag for 50 khz
	movwf   FSR     ;store it in loop
start50 bsf     PORTB,0         ;1  1  set output (pin  6) high
	btfss   PORTA,3         ;2  3 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2  5 has the button on pin 18 been pressed?
	goto    estr50          ;   it's pressed, enter the 10% duty cycle loop
	movlw   6               ;1  6  load up a loop count
	movwf   count           ;1  7  save it 
	;total CYCLE count of Loop is 5 x 3 + 2 = 17
f50k2   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    f50k2           ;2 24  loop back
	nop                     ;1 25  waste a cycle
	bcf     PORTB,0         ;1 26  set output (pin  6) low
	btfss   PORTA,2         ;2 28 has the button on pin 1 been pressed?
	goto    rotat25         ;yes, bail to the rotate to a new loop routine
	movlw   6               ;1 29  load up a loop count
	movwf   count           ;1 30  save it 
	;total CYCLE count of Loop is 5 x 3 + 2 = 17
f50k3   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    f50k3           ;2 47  loop back
	nop                     ;1 48  waste 1 cycle
	goto    start50         ;2 50  keep at it


; 10 % loop for 50 KHz
estr50  bsf     PORTB,0         ;1  1  set output (pin  6) high
	goto    ef50k2          ;2  3  waste 2 cycles
ef50k2  goto    ef50k5          ;2  5  waste 2 cycles
ef50k5  bcf     PORTB,0         ;1  6  set output (pin  6) low
	btfss   PORTA,3         ;2  8  has the button on pin 2 been pressed?
	clrwdt                  ;it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2 10  has the button on pin 18 been released?
	goto    start50         ;it's released, enter the 50% duty cycle loop
	btfss   PORTA,2         ;2 12  has the button on pin 1 been pressed?
	goto    rotat25         ;yes,  bail to the rotate to a new loop routine
	movlw   11              ;1 13  load up a loop count
	movwf   count           ;1 14  save it 
	;total CYCLE count of Loop is 10 x 3 + 2 = 32
ef50k3  decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    ef50k3          ;2 46  loop back
	goto    ef50k4          ;2 48  waste 2 cycles
ef50k4  goto    estr50          ;2 50  keep at it


; rotat25 - a routine to select the next marker frequency
rotat25 ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rotat25 ;no, wait another delay time
			;yes, drift into the 25 Khz loop


;  25 khz loop
;       PORTB bit 0, 20 usec high, 20 usec low (50 cycles hi/lo)
strt251 
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001011'     ; set WDT for 8x (011) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x04            ;load up the flag for 25 khz
	movwf   FSR             ;store it in loop
start25 bsf     PORTB,0         ;1  1  set output (pin  6) high
	btfss   PORTA,3         ;2  3 has the button on pin 2 been pressed?
	clrwdt                   ;     it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2  5 has the button on pin 18 been pressed?
	goto    estr25          ;   it's pressed, enter the 10% duty cycle loop
	movlw   14              ;1  6  load up a loop count
	movwf   count           ;1  7  save it 
	;total CYCLE count of Loop is 13 x 3 + 2 = 41
f25k2   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    f25k2           ;2 48  loop back
	goto    f25k3           ;2 50  waste 2 cycles
f25k3   bcf     PORTB,0         ;1 51  set output (pin  6) low
	btfss   PORTA,2         ;2 53  has the button on pin 1 been pressed?
	goto    rotat10         ;yes, bail to the rotate to a new loop routine
	movlw   14              ;1 54  load up a loop count
	movwf   count           ;1 55  save it 
	;total CYCLE count of Loop is 13 x 3 + 2 = 41
f25k4   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    f25k4           ;2 96  loop back
	goto    f25k5           ;2 98  waste 2 cycles
f25k5   goto    start25         ;2 100  keep at it


; 10 % duty cycle loop, 25 KHz
estr25  bsf     PORTB,0         ;1  1  set output (pin  6) high
	nop                     ;1  2  waste 1 cycle
	goto    ef50ka          ;2  4  waste 2 cycles
ef50ka  goto    ef50kb          ;2  6  waste 2 cycles
ef50kb  goto    ef50kc          ;2  8  waste 2 cycles
ef50kc  goto    ef50kd          ;2 10  waste 2 cycles       
ef50kd  bcf     PORTB,0         ;1 11 set output (pin  6) low
	btfss   PORTA,3         ;2 13 has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2 15 has the button on pin 18 been released?
	goto    start25         ;  it's released, enter the 50% duty cycle loop
	btfss   PORTA,2         ;2 17 has the button on pin 1 been pressed?
	goto    rotat10         ;yes, bail to the rotate to a new loop routine
	;total cycle count of loop is 25 x 3 + 2 = 77
	movlw   26              ;1 18
	movwf   count           ;1 19
efloop1 decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    efloop1         ;2 96  loop back
	goto    ef50ke          ;2 98  loop back
ef50ke  goto    estr25          ;2 100  keep at it


; rotat10 - a routine to select the next marker frequency
rotat10 ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rotat10 ;no, wait another delay time
			;yes, drift into the 10 Khz loop


;  10 khz loop
;       PORTB bit 0, 50 usec high, 50 usec low (125 cycles hi/lo)
strt101 
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001010'     ; set WDT for 4x (010) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x08            ;load up the flag for 10 khz
	movwf   FSR             ;store it in loop
start10 bsf     PORTB,0         ;1   1  set output (pin  6) high
	movlw   d'39'           ;1   2  load up a loop countmovwf   count           ;1   3  save it 
	movwf   count           ;1   3  save the count
	btfss   PORTA,3         ;2   5  has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2   7 has the button on pin 18 been pressed?
	goto    estrt10         ;   it's pressed, enter the 10% duty cycle loop
	;total cycle count of high loop is 38 x 3 + 2 = 116
ohigh   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    ohigh           ;2 123  loop back
	goto    ohighp1         ;2 125  waste 2 cycles
ohighp1 bcf     PORTB,0         ;1 126  set output (pin  6) low
	movlw   d'39'           ;1 127  load up a loop count
	movwf   count           ;1 128  save it 
	;total loop count of high loop is 38 x 3 + 2 = 116
olow    decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    olow            ;2 244  loop back
	goto    herep1          ;2 246  waste 2 cycles
herep1  btfss   PORTA,2         ;2 248  has the button on pin 1 been pressed?
	goto    rotate5         ;yes, bail to the rotate to a new loop routine
	goto    start10         ;2 250  keep at it


; 10 % duty cycle loop for 10 KHz (25 cyc high, 225 low)
estrt10 bsf     PORTB,0         ;1   1  set output (pin  6) high
	movlw   6               ;1   2  load up a loop count
	movwf   count           ;1   3  save it 
	btfss   PORTA,3         ;2   5  has the button on pin 2 been pressed?
	clrwdt                  ;     it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2   7 has the button on pin 18 been released?
	goto    start10         ;  it's released, enter the 50% duty cycle loop
	;total cycle count of high loop is 5 x 3 + 2 = 17
eohigh  decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    eohigh          ;2  24  loop back
	nop                     ;1  25  waste a cycle
	bcf     PORTB,0         ;1  26  set output (pin  6) low
	movlw   73              ;1  27  load up a loop count
	movwf   count           ;1  28  save it 
	;total loop count of high loop is 72 x 3 + 2 = 218
eolow   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    eolow           ;2 246  loop back
eherep1 btfss   PORTA,2         ;2 248  has the button on pin 1 been pressed?
	goto    rotate5         ;yes, bail to the rotate to a new loop routine
	goto    estrt10         ;2 250  keep at it


; rotate5 - a routine to select the next marker frequency
rotate5 ;first, debounce the switch (wait for 40 ms)
	clrwdt          ;clear the watchdog timer
	call    deboun  ;wait 40 ms to debounce the switch
	btfss   PORTA,2 ;has the switch been released?
	goto    rotate5 ;no, wait another delay time
			;yes, drift into the 5 Khz loop


;  5 khz loop
;       PORTB bit 0, 100 usec high, 100 usec low (250 cyc hi/lo)
start51 
	bsf     STATUS,5        ;set the bank bit to 1
	movlw   b'00001001'     ; set WDT for 2x (001) prescale
	movwf   OPTION_REG      ; set the option register
	bcf     STATUS,5        ;set the bank bit to 0
	movlw   0x10    ;load up the flag for 5 khz
	movwf   FSR    ;store it in loop
start5  bsf     PORTB,0         ;1   1  set output (pin  6) high
	movlw   81              ;1   2  load up a loop count
	movwf   count           ;1   3  save it 
	btfss   PORTA,3         ;2   5  has the button on pin 2 been pressed?
	clrwdt                  ;       it's pressed, clear the wdt (no gate)
	btfss   PORTA,1         ;2   7 has the button on pin 18 been pressed?
	goto    estrt5          ;   it's pressed, enter the 10% duty cycle loop
	;total loop count of high loop is 80 x 3 + 2 = 242
ohigh2  decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    ohigh2          ;2 249  loop back
	nop                     ;1 250  waste a cycle
outlow  bcf     PORTB,0         ;1 251  set output (pin  6) low
	movlw   81              ;1 252  load up a loop count
	movwf   count           ;1 253  save it 
	;total loop count of high loop is 80 x 3 + 2 = 242
olow2   decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    olow2           ;2 495  loop back
	nop                     ;1 496  waste a cycle
	btfss   PORTA,2         ;2 498  has the button on pin 1 been pressed?
	goto    rot250          ;yes, bail to the rotate to a new loop routine
	goto    start5          ;2 500  keep at it


; 10 % duty cycle loop for 5 KHz (50 cyc high, 450 cyc lo)
estrt5  bsf     PORTB,0         ;1   1  set output (pin  6) high
	movlw   14              ;1   2  load up a loop count
	movwf   count           ;1   3  save it 
	btfss   PORTA,3         ;2   5  has the button on pin 2 been pressed?
	clrwdt                  ;       it's pressed, clear the wdt (no gate)
	btfsc   PORTA,1         ;2   7 has the button on pin 18 been released?
	goto    start5          ;  it's released, enter the 50% duty cycle loop
	;total cycle count of high loop is 13 x 3 + 2 = 41
eohigh2 decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    eohigh2         ;2  48  loop back
	goto    eoutlow         ;2  50  waste 2 cycles
eoutlow bcf     PORTB,0         ;1  51  set output (pin  6) low
	movlw   148             ;1  52  load up a loop count
	movwf   count           ;1  53  save it 
	;total cycle count of high loop is 147 x 3 + 2 = 443
eolow2  decfsz  count,1         ;1 (2)  decrement the loop count - skip at 0
	goto    eolow2          ;2 496  loop back
	btfss   PORTA,2         ;2 498  has the button on pin 1 been pressed?
	goto    rot250          ;yes, bail to the rotate to a new loop routine
	goto    estrt5          ;2 500  keep at it
	
	
	END


